@@ -58,9 +58,12 @@ send_human	5 flts	send -h timing		user
 #ifdef SYSV4
 #include <sys/stropts.h>
 #endif
+#ifndef M_XENIX
+ /* wait.h is not available on SCO XENIX 386. -- pf@artcom0.north.de */
 #if !defined(CRAY) || CRAY>=60
 #include <sys/wait.h>
 #endif
+#endif
 #include <varargs.h>
 #include <errno.h>
 #ifdef EXTERN_ERRNO
@@ -301,8 +304,12 @@ char **argv;
 #ifdef SYSV3
 	setpgrp();
 #else /* !SYSV3 */
-/*	if (disconnected) setpgrp(0,0);	/* make a new pgrp leader */
-	setpgrp(0,0);	/* make a new pgrp leader */
+#ifdef MIPS_BSD
+	/* required on BSD side of MIPS OS <jmsellen@watdragon.waterloo.edu> */
+#	include <sysv/sys.s>
+	syscall(SYS_setpgrp);
+#endif
+	setpgrp(0,getpid());	/* make a new pgrp leader */
 	ttyfd = open("/dev/tty", O_RDWR);
 	if (ttyfd >= 0) {
 		(void) ioctl(ttyfd, TIOCNOTTY, (char *)0);
@@ -383,8 +390,10 @@ char **argv;
 	close(2);
 	fcntl(0,F_DUPFD,2);	/* duplicate 0 onto 2 */
 
+	/* avoid fflush of cmdfile since this screws up the parents seek ptr */
+	/* There is no portable way to fclose a shared read-stream!!!! */
+	if (cmdfile && (cmdfile != stdin)) (void) close(fileno(cmdfile));
 	if (logfile) (void) fclose(logfile);
-	if (cmdfile) (void) fclose(cmdfile);
 	if (debugfile) (void) fclose(debugfile);
 	/* (possibly multiple) masters are closed automatically due to */
 	/* earlier fcntl(,,CLOSE_ON_EXEC); */
@@ -431,6 +440,9 @@ char **argv;
 	int cooked = FALSE;
 
 	if (argc > 2 && streq(argv[1],"stty")) {
+		extern int ioctled_devtty;
+		ioctled_devtty = TRUE;
+
 		for (i=2;i<argc;i++) {
 			if (streq(argv[i],"raw") ||
 			    streq(argv[i],"-cooked")) {
@@ -1138,6 +1150,11 @@ char **argv;
 		}
 	} else {
 		if (logfile) fclose(logfile);
+		if (*argv[0] == '~') {
+			argv[0] = Tcl_TildeSubst(interp, argv[0]);
+			if (argv[0] == NULL) return(TCL_ERROR);
+		}
+
 		if (NULL == (logfile = fopen(argv[0],"a"))) {
 			tcl_error("%s: %s",argv[0],sys_errlist[errno]);
 			logfile_all = FALSE;
@@ -1192,6 +1209,10 @@ char **argv;
 			return(TCL_ERROR);
 		}
 		if (debugfile) fclose(debugfile);
+		if (*argv[0] == '~') {
+			argv[0] = Tcl_TildeSubst(interp, argv[0]);
+			if (argv[0] == NULL) return(TCL_ERROR);
+		}
 		if (NULL == (debugfile = fopen(*argv,"a"))) {
 			tcl_error("%s: %s",*argv,sys_errlist[errno]);
 			return(TCL_ERROR);
@@ -1331,13 +1352,12 @@ char **argv;
 {
 	int master;
 	static char waitstr[80];   /* should be long enough for two ints */
-	int status;
 	struct f *f;
 	/* if your C compiler bombs here, define NO_PID_T in Makefile */
 #ifdef NO_PID_T
-	int pid;
+	int pid = 0;	/* 0 indicates no error occurred (yet) */
 #else
-	pid_t pid;
+	pid_t pid = 0;	/* ditto */
 #endif
 
 	argc--; argv++;
@@ -1361,30 +1381,39 @@ char **argv;
 	}
 
 	if (!(f = fd_to_f(master,"wait"))) return(TCL_ERROR);
-	if (f->flags & FD_SYSWAITED) {
-		if (f->flags & FD_CLOSED) f->flags = 0;
-		return(f->wait);
-	}
-	for (;;) {
-		int i;
 
-		pid = wait(&status);
-		if (pid == f->pid) break;
-		/* oops, wrong pid */
-		for (i=0;i<=fd_max;i++) {
-			if (fs[i].pid == pid) break;
+	if (!(f->flags & FD_SYSWAITED)) {
+#ifdef NOWAITPID
+		int status;
+		for (;;) {
+			int i;
+
+			pid = wait(&status);
+			if (pid == f->pid) break;
+			/* oops, wrong pid */
+			for (i=0;i<=fd_max;i++) {
+				if (fs[i].pid == pid) break;
+			}
+			if (i>fd_max) {
+				debuglog("wait found unknown pid %d\r\n",pid);
+				continue;	/* drop on floor */
+			}
+			fs[i].flags |= FD_SYSWAITED;
+			fs[i].wait = status;
 		}
-		if (i>fd_max) {
-			debuglog("wait found unknown pid %d\r\n",pid);
-			continue;	/* drop on floor */
-		}
-		fs[i].flags |= FD_SYSWAITED;
-		fs[i].wait = status;
+		f->wait = status;
+#else
+		pid = waitpid(f->pid,&f->wait,0);
+#endif
+		f->flags |= FD_SYSWAITED;
 	}
 	if (f->flags & FD_CLOSED) f->flags = 0;
-
+	f->flags |= FD_USERWAITED;
+#ifndef WEXITSTATUS
+#define WEXITSTATUS(x) x
+#endif
 	/* non-portable assumption that pid_t is an int */
-	sprintf(waitstr,"%d %d",pid,(pid == -1)?errno:status);
+	sprintf(waitstr,"%d %d",f->pid,(pid == -1)?errno:WEXITSTATUS(f->wait));
 	/* return {pid status} or {-1 errno} */
 	Tcl_Return(interp,waitstr,TCL_STATIC);
 	return(TCL_OK);
@@ -1460,6 +1489,11 @@ char **argv;
 	/* UNIX Network Programming, p. 79-80) */
 	/* second child process continues as daemon */
 #else /* !SYSV3 */
+#ifdef MIPS_BSD
+	/* required on BSD side of MIPS OS <jmsellen@watdragon.waterloo.edu> */
+#	include <sysv/sys.s>
+	syscall(SYS_setpgrp);
+#endif
 	setpgrp(0,getpid());	/* put process in our own pgrp */
 	ttyfd = open("/dev/tty", O_RDWR);
 	if (ttyfd >= 0) {
