@@ -5,6 +5,10 @@ Written by: Don Libes, NIST, 2/6/90
 Design and implementation of this program was paid for by U.S. tax
 dollars.  Therefore it is public domain.  However, the author and NIST
 would appreciate credit if this program or parts of it are used.
+
+$Revision: 1.1 $
+$Date: 1992/02/21 19:05:45 $
+
 */
 
 #include <sys/types.h>
@@ -23,6 +27,7 @@ extern int errno;
 #include <ctype.h>	/* for isspace */
 #include <time.h>	/* for time(3) */
 #include <setjmp.h>
+#include <sys/wait.h>
 #include "tcl.h"
 extern char *tclRegexpError;	/* declared in tclInt.h */
 #include "regexp.h"
@@ -469,7 +474,7 @@ int me;
 	}
 
 	*mcount = 0;	/* initially empty */
-	for (i=0,ec=*ecases; i < *ecases_inuse ;i++,ec++) {
+	for (i=0,ec= *ecases; i < *ecases_inuse ;i++,ec++) {
 		int j;
 
 		if (ec->m == SPAWN_ID_ANY) continue;
@@ -1507,6 +1512,7 @@ init_trap()
 	signals[SIGUSR2].name = "SIGUSR2";
 #endif
 
+#if 0
 #ifdef HPUX
 	/* initially forced to catch & discard SIGCLD to collect wait status */
 	(void) Tcl_Eval(interp,"trap SIG_DFL SIGCHLD",0,(char **)0);
@@ -1517,6 +1523,7 @@ init_trap()
 	/* them both, but expect can only handle one, and it handles the */
 	/* "wrong" one, first */
 #endif
+#endif
 }
 
 /* reserved to us if name begins with asterisk */
@@ -1568,6 +1575,8 @@ void
 generic_sighandler(sig)
 int sig;
 {
+	int proc_valid = TRUE;
+
 	debuglog("generic_sighandler: handling signal(%d)\r\n",sig);
 
 	if (sig < 0 || sig >= NSIG) {
@@ -1583,42 +1592,41 @@ int sig;
 		int rc;
 
 #ifdef REARM_SIG
-		/* avoid reinstalling signal 0 which can never be generated */
-		/* twice since it only occurs at program termination */
-		if (sig != 0) {
 #ifdef SYSV3
-			/* assume no wait() occurs between SIGCLD and */
-			/* this code */
-			if (sig == SIGCLD) {
-				int i, pid;
-				int status;
-				extern int fd_max;
+		/* assume no wait() occurs between SIGCLD and */
+		/* this code */
+		if (sig == SIGCLD) {
+			int i, pid;
+			int status;
+			extern int fd_max;
 
-				pid = wait(&status);
-				for (i=0;i<=fd_max;i++) {
-					if (fs[i].pid == pid) break;
-				}
-				if (i>fd_max) {
-					debuglog("received SIGCLD from unknown pid %d\r\n",pid);
-				} else {
-					fs[i].flags = FD_SYSWAITED;
-					fs[i].wait = status;
-				}
+			pid = wait(&status);
+			for (i=0;i<=fd_max;i++) {
+				if (fs[i].pid == pid) break;
+			}
+			if (i>fd_max || !(fs[i].flags & FD_VALID)) {
+				debuglog("received SIGCLD from unknown pid %d\r\n",pid);
+				proc_valid = FALSE;
+			} else {
+				fs[i].flags |= FD_SYSWAITED;
+				fs[i].wait = status;
 			}
-#endif
-			signal(sig,generic_sighandler);
 		}
 #endif
+		if (sig != 0) signal(sig,generic_sighandler);
+#endif
 
 		debuglog("generic_sighandler: Tcl_Eval(%s)\r\n",signals[sig].action);
-		rc = Tcl_Eval(interp,signals[sig].action,0,(char **)0);
-		if (rc != TCL_OK) {
+		if (proc_valid) {
+		  rc = Tcl_Eval(interp,signals[sig].action,0,(char **)0);
+		  if (rc != TCL_OK) {
 		    errorlog("caught %s (%d): error in command: %s\r\n",
 			signal_to_string(sig),sig,signals[sig].action);
 		    if (rc != TCL_ERROR) errorlog("Tcl_Eval = %d\r\n",rc);
 		    if (*interp->result != 0) {
 			errorlog("%s\r\n",interp->result);
 		    }
+	          }
 		}
 	}
 
@@ -1746,10 +1754,12 @@ char **argv;
 		}
 */
 
+#if 0
 #ifdef HPUX
 		if (sig == SIGCLD && streq(action,"SIG_DFL")) {
 			action = "";
 		}
+#endif
 #endif
 
 		if (sig == SIGALRM) {
@@ -1764,16 +1774,17 @@ char **argv;
 		if (signals[sig].action) free(signals[sig].action);
 
 		if (streq(action,"SIG_DFL")) {
-			sig_reset(sig);
+			if (sig != 0) sig_reset(sig);
 		} else {
 			len = 1 + strlen(action);
 			if (0 == (signals[sig].action = malloc(len))) {
 				tcl_error("trap: malloc failed");
-				sig_reset(sig);
+				if (sig != 0) sig_reset(sig);
 				rc = TCL_ERROR;
 				break;
 			}
 			memcpy(signals[sig].action,action,len);
+			if (sig == 0) continue;
 			if (streq(action,"SIG_IGN")) {
 				signal(sig,SIG_IGN);
 			} else signal(sig,generic_sighandler);
