@@ -25,18 +25,22 @@ int errno;	/* some but not all systems require this */
 #endif
 
 /* yes, I have a weak mind */
-#define streq(x,y)	(0 == (strcmp((x),(y))))
+#define streq(x,y)	(0 == strcmp((x),(y)))
 
 #ifdef NO_MEMCPY
 #define memcpy(x,y,len) bcopy(y,x,len)
 #endif
 
 #ifndef TERM
-#ifdef SYSV3
-#define TERM termios
-#else
-#define TERM sgttyb
-#endif
+#  ifdef SYSV3
+#    ifdef HPUX
+#      define TERM termio
+#    else
+#      define TERM termios
+#    endif
+#  else
+#      define TERM sgttyb
+#  endif
 #endif
 
 typedef struct TERM exp_tty;
