@@ -5,6 +5,10 @@ Written by: Don Libes, NIST, 2/6/90
 Design and implementation of this program was paid for by U.S. tax
 dollars.  Therefore it is public domain.  However, the author and NIST
 would appreciate credit if this program or parts of it are used.
+
+$Revision: 1.2 $
+$Date: 1992/02/21 18:47:50 $
+
 */
 
 #include <stdio.h>
@@ -23,6 +27,11 @@ extern int errno;
 #endif
 #ifdef HPUX
 #include <unistd.h>
+#include <sys/ptyio.h>
+#endif
+#ifdef _AIX
+/* AIX has some unusual definition of FD_SET */
+#include <sys/select.h>
 #endif
 #include "tcl.h"
 #include "translate.h"
@@ -164,6 +173,10 @@ Tcl_Interp *interp;
 int argc;
 char **argv;
 {
+#ifdef HPUX
+	fd_set excep;
+	int enable = 1;
+#endif
 	fd_set rdrs;
 	int rc, wc;		/* return codes for various system calls */
 
@@ -286,9 +299,17 @@ char **argv;
 		finish(TCL_ERROR);
 	}
 
-	tty_changed = tty_raw_noecho(&tty_old,&was_raw,&was_echo);
+	if (user_in == 0) {
+		tty_changed = tty_raw_noecho(&tty_old,&was_raw,&was_echo);
+	}
 
 	FD_ZERO(&rdrs);
+#ifdef HPUX
+	FD_ZERO(&excep);
+
+	/* Turn on trapping of close, open and ioctl requests from the slave.*/
+	ioctl(master, TIOCTRAP, &enable);
+#endif
 
 	for (;;) {
 		int match;
@@ -302,7 +323,14 @@ char **argv;
 		    || master_match_in_progress) {
 			FD_SET(user_in,&rdrs);
 			FD_SET(master,&rdrs);
+#ifdef HPUX
+			FD_SET(user_in,&excep);
+			FD_SET(master,&excep);
+/*debuglog("Calling select #1, %d, %ld\r\n", maxfds, rdrs.fds_bits[0]);*/
+			if (-1 == select(maxfds,&rdrs,(fd_set *)0,&excep,
+#else
 			if (-1 == select(maxfds,&rdrs,(fd_set *)0,(fd_set *)0,
+#endif
 						(struct timeval *)0)) {
 				/* window refreshes trigger EINTR, ignore */
 				if (errno == EINTR) continue;
@@ -324,6 +352,53 @@ char **argv;
 
 		if (m->size == 0 || master_match_in_progress) {
 			/* received from master, send to user */
+
+#ifdef HPUX
+			/*
+			 * Exceptional condition from slave process on pty.
+			 * This could be either a close or an open or an ioctl
+			 * request that is not handled by the termio driver on
+			 * the slave side of the pty.
+			 *
+			 * On HP-UX this is the only way to tell that the other
+			 * process exited.  We set up trapping above to trap
+			 * close actions on the pty.  When the process on the
+			 * slave side exits the file descriptor is closed.
+			 * On BSD4.3 and Sys5 a read would return
+			 * zero bytes read, so this code is not needed.
+			 *
+			 * This code provided by Bob Proulx and Jeff
+			 * Okamoto of HP
+			 */
+			if (FD_ISSET(master, &excep)) {
+
+				struct request_info ioctl_info;
+				if (ioctl(master,TIOCREQCHECK,&ioctl_info) < 0) {
+					debuglog("ioctl error on TIOCREQCHECK: %d", errno);
+					break;
+				}
+				if (ioctl_info.request == TIOCCLOSE) {
+
+/* The process on the slave side closed the pty.  This probably means that it
+ exited.  It could be that the process just closed the file descriptor.
+ The close is not blocked on the slave side.  If the master side is closed
+ the handshake will be completed automatically.  Since there is no need to
+ complete the handshake we return here.
+ */
+					break;	/* EOF from slave process. */
+				}
+/* Handshake the ioctl or the open request.  We don't know what the request
+ was but we return it as successfully completed even though we did nothing.
+ This was probably some type of modem control which would mean something on a
+ real tty but is meaningless on a pty.  The application is faked-out and that
+ is the purpose of this code anyway.
+ */
+				if (ioctl(master, TIOCREQSET, &ioctl_info) < 0)
+					debuglog("ioctl error on TIOCREQSET after ioctl or open on slave: %d", errno);
+			}
+#endif /*HPUX*/
+
+			/* received from master, send to user */
 			if (FD_ISSET(master, &rdrs)) {
 				if (0 >= (rc = read(master, m->buffer,
 						m->msize-m->size))) {
@@ -432,12 +507,12 @@ char **argv;
 
 			m->buffer[m->size] = 0;
 
-/* If expect has left characters in buffer, it has already echoed them */
-/* to the screen, thus we must prevent them being rewritten.  Unfortunately */
-/* this gives the possibility of matching chars that have already been */
-/* output, but we do so since the user could have avoided it by flushing the */
-/* output buffers directly. */
-/* Hopefully, this code should still be fast for when m->printed == 0 */
+/* If expect has left characters in buffer, it has already echoed them to the
+ screen, thus we must prevent them being rewritten.  Unfortunately this gives
+ the possibility of matching chars that have already been output, but we do so
+ since the user could have avoided it by flushing the output buffers directly.
+ Hopefully, this code should still be fast for when m->printed == 0
+*/
 			skip = 0;
 			write_count = match;
 			if (m->printed) {
@@ -463,7 +538,7 @@ char **argv;
 
 			m->size -= match;
 			if (m->size != 0) {
-			   memcpy(u->buffer,u->buffer+match,u->size);
+			   memcpy(m->buffer,m->buffer+match,m->size);
 			}
 			m->buffer[m->size] = '\0';
 		}
