@@ -23,14 +23,24 @@ it successfully, please mail back the changes to me.  - Don
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
-/*#include <sys/file.h>*/
+
+#ifndef M_XENIX
 #include <sys/wait.h>
+#else
+#ifndef WEXITSTATUS
+#define WEXITSTATUS(stat) (((*((int *) &(stat))) >> 8) & 0xff)
+#endif
+  /* <sys/wait.h> is not available on SCO XENIX 386 -- pf@artcom0.north.de */
+#endif
+
+#include <ctype.h>
 #include <setjmp.h>
 #include <errno.h>
 #include "tcl.h"
 #include "translate.h"
 #include "global.h"
 #include "command.h"
+#include "term.h"
 
 #define min(x,y)	(((x)<(y))?(x):(y))
 
@@ -176,11 +186,11 @@ clean_up_after_child(master)
 int master;
 {
 /* should really be recoded using the common wait code in command.c */
-	union wait wait_status;
+	int wait_status;
 
 	/* first, find out why it died */
 	(void) wait(&wait_status);
-	if (wait_status.w_T.w_Retcode == USER_DIED) bye(0);
+	if (WEXITSTATUS(wait_status) == USER_DIED) bye(0);
 	/* else process died */
 	deferred_interrupt = FALSE;
 	fd_close(master);
@@ -465,14 +475,18 @@ char **argv;
 
 			m->size -= match;
 			if (m->size != 0) {
-			   memcpy(u->buffer,u->buffer+match,u->size);
+			   memcpy(m->buffer,m->buffer+match,m->size);
 			}
 			m->buffer[m->size] = '\0';
 		}
 	   }
 	} else { /* parent */
 		debuglog("fork = %d\r\n",pid);
+#ifdef SIGCHLD
 		signal(SIGCHLD,sigchld_handler);
+#else
+		signal(SIGCLD,sigchld_handler);
+#endif
 	restart:
 		tty_changed = tty_raw_noecho(&tty_old,&was_raw,&was_echo);
 
