@@ -8,7 +8,11 @@ would appreciate credit if this program or parts of it are used.
 */
 
 #include <stdio.h>
+#ifndef M_XENIX
+  /* netdb.h is not available on SCO XENIX 386, unless you bought
+     the TCP/IP Development Package.  <pf@artcom0.north.de> 26-Jan-92 : */
 #include <netdb.h>
+#endif
 #include <sys/types.h>
 #ifdef CRAY
 #include <sys/inode.h>	/* Cray 5.1 needs this, I don't know why */
@@ -79,6 +83,7 @@ int is_noecho = FALSE;
 extern int disconnected;
 
 exp_tty tty_current, tty_cooked;
+int ioctled_devtty = FALSE;
 
 /* if set == 1, set it to raw, else unset it */
 void
@@ -155,6 +160,7 @@ int *was_raw, *was_echo;
 		bye(-1);
 	}
 
+	ioctled_devtty = TRUE;
 	return(1);
 }
 
@@ -184,6 +190,7 @@ int *was_raw, *was_echo;
 		errorlog("ioctl(noraw): %s\r\n",sys_errlist[errno]);
 		bye(-1);
 	}
+	ioctled_devtty = TRUE;
 
 	return(1);
 }
@@ -206,6 +213,7 @@ int echo;
 	is_noecho = !echo;
 	tty_current = *tty;
 	debuglog("tty_set: raw = %d, echo = %d\r\n",is_raw,!is_noecho);
+	ioctled_devtty = TRUE;
 }	
 
 init_tty()
@@ -244,7 +252,7 @@ int status;
 	flush_streams();	/* need to do this before tty is reset, */
 	/* because formatting earlier was done sensitive to tty mode */
 
-	if (!disconnected && !forked && isatty(0))
+	if (!disconnected && !forked && isatty(0) && ioctled_devtty)
 		tty_set(&exp_tty_original,0,0);
 
 	/* all other files either don't need to be flushed or will be
