@@ -1,4 +1,4 @@
-/* $Date: 92/02/16 08:41:45 $    $Revision: 1.3 $  by $Author: joe $  */
+/* $Date: 92/03/10 15:35:13 $    $Revision: 1.4 $  by $Author: joe $  */
 /* 
   Copyright (C) 1991 by the Massachusetts Institute of Technology
 
@@ -203,14 +203,14 @@ void CmFigure::Widgetize() {
 Widget CmFigure::widgetize()  {
   XrmBinding bindings[MAX_NUM_QUARKS];
   XrmQuark quarks[MAX_NUM_QUARKS];
-  
+  char widget_name[256];
+
   Widget widget_parent_of_this = 
     widget_parent ? widget_parent : parent->widget;
 
   XrmDatabase database = XtDatabase(XtDisplay(widget_parent_of_this));
 
-  CmString widget_name = "CDS_widget_";
-  widget_name += form("%d", counter);
+  sprintf(widget_name, "CDS_widget_%d", counter);
 
   XrmStringToBindingQuarkList(widget_name, bindings, quarks);
   bindings[0] = XrmBindLoosely;
@@ -249,7 +249,7 @@ Widget CmFigure::widgetize()  {
 
   counter++;
 
-  widget = XtVaCreateWidget(widget_name.Chars(),
+  widget = XtVaCreateWidget(widget_name,
 			    cmfigure_class->GetWidgetClass(), 
 			    widget_parent_of_this,
 			    XmNleftAttachment, XmATTACH_POSITION,
@@ -308,6 +308,18 @@ const CmString & CmFigure::GetResource(const CmString &tok) {
  CmFigure ChangeResource 
   Changes the value of a certain resource
 **************************/
+void CmFigure::ChangeResourceInt(const CmString &name, int value) {
+  char string[32];
+  sprintf(string, "%d", value);
+  ChangeResource(name, string);
+}
+
+void CmFigure::ChangeResourceDouble(const CmString &name, double value) {
+  char string[64];
+  sprintf(string, "%lf", value);
+  ChangeResource(name, string);
+}
+
 void CmFigure::ChangeResource(const CmString &name,  
 			      const CmString &value) {
   if (cmfigure_class->Default(name) == value)
@@ -547,12 +559,14 @@ void CmFigure::GetCmFigureDefinition(CmString &def_string,
 
 // Resize
 void CmFigure::Resize(int x, int y, int width, int height) {
+  char string[64];
   Position leftpos = x, toppos = y;
   Position rightpos = x + width, botpos = y + height;
-  ChangeResource(XmNleftPosition, form("%d", leftpos));
-  ChangeResource(XmNtopPosition, form("%d",toppos));
-  ChangeResource(XmNrightPosition, form("%d", rightpos));
-  ChangeResource(XmNbottomPosition, form("%d", botpos));
+
+  ChangeResourceInt(XmNleftPosition, leftpos);
+  ChangeResourceInt(XmNtopPosition, toppos);
+  ChangeResourceInt(XmNrightPosition, rightpos);
+  ChangeResourceInt(XmNbottomPosition, botpos);
   
   if (widget) {
     XtVaSetValues(widget,
