@@ -1,4 +1,4 @@
-/* $Date: 92/02/16 08:44:01 $    $Revision: 1.2 $  by $Author: joe $  */
+/* $Date: 92/03/10 15:35:39 $    $Revision: 1.3 $  by $Author: joe $  */
 /* 
   Copyright (C) 1991 by the Massachusetts Institute of Technology
 
@@ -511,17 +511,15 @@ void CmSnapResolution(Widget, CmFigureHandler* handler, XtPointer) {
   resdialog->AddPane("resolution","", CMDIALOG_ATTACH_NONE);
   resdialog->AddItem(CMDIALOG_TEXTSTRING,"resolution","X Snap");
   resdialog->AddItem(CMDIALOG_TEXTSTRING,"resolution","Y Snap");
-  CmString xsnap, ysnap;
-  xsnap = (form("%d",handler->Xscale));
-  ysnap = (form("%d",handler->Yscale));
-  cmdebug << xsnap << ", " << ysnap << "\n";
 
-  resdialog->AddOption("resolution","X Snap",
-			    CMDIALOG_VALUE,
-			    (XtArgVal)( xsnap.Chars() ));
-  resdialog->AddOption("resolution","Y Snap",
-			    CMDIALOG_VALUE,
-			    (XtArgVal)( ysnap.Chars() ));
+  char string[16];
+  sprintf(string, "%d", handler->Xscale);
+  resdialog->AddOption("resolution","X Snap", CMDIALOG_VALUE, 
+		       (XtArgVal) string);
+
+  sprintf(string, "%d", handler->Yscale);
+  resdialog->AddOption("resolution","Y Snap", CMDIALOG_VALUE,
+		       (XtArgVal) string);
   
   resdialog->ManageItems();
   resdialog->PopupDialog();
